/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#include "pch.h"
#include "application.h"
#include "checkpointobject.h"
#include "level.h"
#include "playerobject.h"

CheckpointObject::CheckpointObject(Layer& layer,
								   const TiXmlElement* p_xml,
								   const ParameterMap& gameParams)
	: Object(layer, p_xml, gameParams)
{
	// Typ lesen
	type = p_xml->Attribute("type");

	// Sprites laden
	sprites = app.getTextureManager().getResourceHandle("sprites/checkpoint.png");
	level.addResourceToCache(sprites);

	if(type == "Finish")
	{
		visibilityRadius = 1.0f;
		b2BodyDef bodyDef;
		b2Body& body = createBody(bodyDef);
		b2PolygonDef polygonDef;
		polygonDef.SetAsBox(0.5f, 1.5f);
		polygonDef.isSensor = true;
		body.CreateShape(&polygonDef);
	}

	if(getGameMode() != GM_TIME_1P && getGameMode() != GM_TIME_2P) kill();

	// Initialisierung fertigstellen
	finishInit(p_xml);
}

void CheckpointObject::render()
{
	app.setBlending(true);
	app.setTexturing(true);
	sprites->bind();

	if(type == "Finish") drawBodyQuad(0, Vec2i(0, 0), Vec2i(192, 256), Vec2f(2.25f, 3.0f));
}

void CheckpointObject::onCollision(const Contact& contact)
{
	if(const PlayerObject* p_player = dynamic_cast<const PlayerObject*>(contact.p_otherObject))
	{
		// Spieler hat den Checkpoint erreicht!
		reachedBy.insert(p_player);
	}
}

bool CheckpointObject::wasReachedByPlayer(const PlayerObject* p_player) const
{
	return reachedBy.find(p_player) != reachedBy.end();
}
