/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#include "pch.h"
#include "application.h"
#include "creditsstate.h"
#include "transitions.h"

CreditsState::CreditsState(Application& app)
	: State("Credits", app)
	, xPos(6.0f)
	, yPos(25.0f)
	, speed(1.875f)
	, shocked(false)
{
	// Schriftart, Gradient, Monster-Bild und Schrei-Sound laden
	font = app.getFontManager().getResourceHandle("fonts/arial.fnt");
	bigFont = app.getFontManager().getResourceHandle("fonts/bellgothic.fnt");
	gradient = app.getTextureManager().getResourceHandle("sprites/gradient.jpg");
	monster = app.getTextureManager().getResourceHandle("sprites/monster.png");
	scream = app.getSoundManager().getResourceHandle("sounds/scream.ogg");
}

void CreditsState::onEnter(const ParameterMap& params)
{
	yPos = 25.0f;
	shocked = false;
}

void CreditsState::onLeave()
{

}

void CreditsState::onGotFocus()
{

}

void CreditsState::onLostFocus()
{

}

void CreditsState::update()
{
	yPos -= (speed * (app.isKeyDown(SDLK_SPACE) ? 50.0f : 1.0f)) * app.getUpdateStepSize();
	if(yPos < -height)
	{
		yPos = 25.0f;
		shocked = false;
	}

	if(app.wasKeyPressed(SDLK_ESCAPE))
	{
		app.popState();
		app.startTransition(new CubeTransition(app, 0.75f));
		return;
	}
}

void CreditsState::render()
{
	static const RGBA textCol(0.15f, 0.65f, 0.82f);
	static const RGBA headingCol(1.0f, 1.0f, 1.0f);

	app.clearViewport(RGBA::BLACK);

	// Projektionsmatrix einstellen
	glMatrixMode(GL_PROJECTION);
	glOrtho(0.0, 32.0, 24.0, 0.0, 0.0, 1.0);

	glMatrixMode(GL_MODELVIEW);

	float localY = yPos;

	bigFont->drawText(Vec2f(xPos - 0.5f, localY), 1.25f, headingCol, "Rollmorad Guhase 2010 Team");
	localY += 2.0f;

	font->drawText(Vec2f(xPos, localY), 1.0f, textCol,
				   "Jan Hapke\n"
				   "Wilhelm Mailnder\n"
				   "David Scherfgen");
	localY += 5.5f;

	bigFont->drawText(Vec2f(xPos - 0.5f, localY), 1.25f, headingCol, "Verwendete Bibliotheken");
	localY += 2.0f;

	font->drawText(Vec2f(xPos, localY), 1.0f, textCol,
				   "Box2D\n"
				   "Ogg Vorbis\n"
				   "OpenAL\n"
				   "OpenGL\n"
				   "SDL\n"
				   "SDL_Image\n"
				   "TinyXML");
	localY += 9.5f;

	bigFont->drawText(Vec2f(xPos - 0.5f, localY), 1.25f, headingCol, "Besonderer Dank an");
	localY += 2.0f;

	font->drawText(Vec2f(xPos, localY), 1.0f, textCol,
				   "www.soundsnap.com\n"
				   "Edmund Drcker\n"
				   "Jakob Mierscheid\n"
				   "Friedrich Nagelmann\n"
				   "Frank \"Grimey\" Grimes\n"
				   "");
	localY += 20.0f;

	const char* p_additionalTexts[] = { "Das war's!",
										"Wirklich jetzt.",
										"Hier gibt es nichts mehr zu sehen!",
										"Du verschwendest nur deine Zeit ...",
										"Laaaaangweilig!",
										"Willst du nicht lieber spielen?",
										"Oder fernsehen?",
										"Was kommt denn so im Fernsehen?",
										"...",
										"Schnarch ...",
										"",
										"Ich bin kurz eingeschlafen.",
										"Immer noch da?!",
										"Mir fllt nichts mehr ein ...",
										"Na gut!",
										"Wenn du schon so hartnckig bist, ...",
										"... dann erzhle ich dir halt was.",
										"Eine fantastische Geschichte ...",
										"... von Hass und Rachsucht ...",
										"... weit ber das Grab hinaus!",
										"Es war einmal, vor langer Zeit ...",
										"... wie war das noch?",
										"Ich hab's vergessen!",
										"[SHOCK]",
										"Haha! Erschrocken?",
										"Das hat man dann davon!",
										"Jetzt reicht's aber wirklich.",
										"Tschss!" };

	for(int i = 0; i < sizeof(p_additionalTexts) / sizeof(p_additionalTexts[0]); ++i)
	{
		const std::string text(p_additionalTexts[i]);
		if(text == "[SHOCK]")
		{
			if(localY > 4.0f && localY < 24.0f)
			{
				// Monster rendern
				app.setTexturing(true);
				app.setBlending(true);
				monster->bind();
				glPushMatrix();
				glTranslatef(16.0f, 12.0f, 0.0f);
				const float scale = 1.0f + 0.25f * sinf(5.0f * (localY - 24.0f));
				glScalef(scale, scale, 1.0f);
				glBegin(GL_QUADS);
				glColor4f(1.0f, 1.0f, 1.0f, localY - 10.0f);
				glTexCoord2i(0, 0);
				glVertex2f(-12.0f, -10.0f);
				glTexCoord2i(512, 0);
				glVertex2f(12.0f, -10.0f);
				glTexCoord2i(512, 512);
				glVertex2f(12.0f, 10.0f);
				glTexCoord2i(0, 512);
				glVertex2f(-12.0f, 10.0f);
				glEnd();
				glPopMatrix();

				if(!shocked)
				{
					// Schrei abspielen
					scream->createInstance();
					shocked = true;
				}
			}
		}
		else if(localY > -1.0f && localY < 24.0f)
		{
			bigFont->drawText(Vec2f(xPos, localY), 1.25f, headingCol, text);
		}

		localY += 20.0f;
	}

	height = localY - yPos;

	app.setTexturing(true);
	app.setBlending(true, GL_DST_COLOR, GL_ZERO);
	gradient->bind();
	glBegin(GL_QUADS);
	glColor4fv(RGBA::WHITE);
	glTexCoord2i(0, 0);
	glVertex2f(0.0f, 0.0f);
	glTexCoord2i(512, 0);
	glVertex2f(32.0f, 0.0f);
	glTexCoord2i(512, 512);
	glVertex2f(32.0f, 24.0f);
	glTexCoord2i(0, 512);
	glVertex2f(0.0f, 24.0f);
	glEnd();
}
