/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#include "pch.h"
#include "cubetransition.h"

CubeTransition::CubeTransition(Application& app,
							   float duration)
	: Transition(app, true, duration)
{

}

void CubeTransition::render(GLuint oldImageTextureID,
							GLuint newImageTextureID)
{
	const float t = easeInEaseOut(time / duration);

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	gluPerspective(90.0, 1.0, 0.1, 100.0);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	gluLookAt(0.0, 0.0, -2.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
	glRotatef(90.0f * t, 0.0f, 1.0f, 0.0f);

	glClearColor(0.0, 0.0, 0.0, 1.0);
	glClear(GL_COLOR_BUFFER_BIT);

	glEnable(GL_TEXTURE_2D);
	glEnable(GL_CULL_FACE);

	// vordere Wrfelseite zeichnen
	glBindTexture(GL_TEXTURE_2D, oldImageTextureID);
	glBegin(GL_QUADS);
	glColor3f(1.0f - t, 1.0f - t, 1.0f - t);
	glTexCoord2i(screenResolution.x, 0);
	glVertex3i(-1, 1, -1);
	glTexCoord2i(0, 0);
	glVertex3i(1, 1, -1);
	glTexCoord2i(0, screenResolution.y);
	glVertex3i(1, -1, -1);
	glTexCoord2i(screenResolution.x, screenResolution.y);
	glVertex3i(-1, -1, -1);
	glEnd();

	// linke Wrfelseite zeichnen
	glRotated(-90.0, 0.0, 1.0, 0.0);
	glBindTexture(GL_TEXTURE_2D, newImageTextureID);
	glBegin(GL_QUADS);
	glColor3f(t, t, t);
	glTexCoord2i(screenResolution.x, 0);
	glVertex3i(-1, 1, -1);
	glTexCoord2i(0, 0);
	glVertex3i(1, 1, -1);
	glTexCoord2i(0, screenResolution.y);
	glVertex3i(1, -1, -1);
	glTexCoord2i(screenResolution.x, screenResolution.y);
	glVertex3i(-1, -1, -1);
	glEnd();

	glDisable(GL_TEXTURE_2D);
	glDisable(GL_CULL_FACE);

	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
}
