/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#include "pch.h"
#include "application.h"
#include "dummyobject.h"
#include "level.h"

DummyObject::DummyObject(Layer& layer,
						 const TiXmlElement* p_xml,
						 const ParameterMap& gameParams)
	: Object(layer, p_xml, gameParams)
	, frame(0.0f)
{
	// Sprites laden
	sprites = app.getTextureManager().getResourceHandle("sprites/dummy.png");
	level.addResourceToCache(sprites);

	// Body erzeugen
	b2BodyDef bodyDef;
	b2Body& body = createBody(bodyDef);

	// Shape erzeugen
	b2CircleDef circleDef;
	circleDef.radius = 0.4f;
	circleDef.density = 1.0f;
	body.CreateShape(&circleDef);
	body.SetMassFromShapes();

	// Initialisierung fertigstellen
	finishInit(p_xml);
}

void DummyObject::update()
{
	frame += 8.0f * app.getUpdateStepSize();

	// Tastatursteuerung
	b2Vec2 force(0.0f, 0.0f);
	if(app.isKeyDown(SDLK_a)) force.x = -20.0f;
	if(app.isKeyDown(SDLK_d)) force.x = 20.0f;
	if(app.isKeyDown(SDLK_w)) force.y = -20.0f;
	if(app.isKeyDown(SDLK_s)) force.y = 20.0f;
	getBody(0).ApplyForce(force, getBody(0).GetWorldCenter());

	// Layer wechseln
	if(app.wasKeyPressed(SDLK_1)) setLayer(*level.getLayerByID("3"));
	if(app.wasKeyPressed(SDLK_2)) setLayer(*level.getLayerByID("4"));
}

void DummyObject::render()
{
	app.setBlending(true);
	app.setTexturing(true);
	sprites->bind();

	glPushMatrix();

	const Vec2f position(getPosition());
	glTranslatef(position.x, position.y, 0.0f);
	glRotatef(rad2Deg(getAngle()), 0.0f, 0.0f, 1.0f);

	const int tx = static_cast<int>(2.0f + 1.99f * sin(frame)) * 64;
	glColor4fv(RGBA::WHITE);
	glBegin(GL_QUADS);
	glTexCoord2i(tx, 0);
	glVertex2f(-0.5f, -0.5f);
	glTexCoord2i(tx + 64, 0);
	glVertex2f(0.5f, -0.5f);
	glTexCoord2i(tx + 64, 64);
	glVertex2f(0.5f, 0.5f);
	glTexCoord2i(tx, 64);
	glVertex2f(-0.5f, 0.5f);
	glEnd();

	glPopMatrix();
}
