/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#ifndef FILELOCATOR_H_INCLUDED
#define FILELOCATOR_H_INCLUDED

//Verzeichnisse registrieren, in denen nach dateien gesucht wird.
//abstraktion von dateizugriffen in unterverzeichnissen.

class FileLocator
	: NonCopyable
{
public:
	//verzeichnis registrieren.
	//pfad im string, prioritt als int. default wert 0, hhere prioritt: niedrigerer wert.
	void addSearchPath(const std::string& path, int priority = 0);

	// Pfad entfernen
	void removeSearchPath(const std::string& path);

	//pfad neue prioritt verpassen
	void changePathPriority(const std::string& path, int priority = 0);

	//vollstndigen(relativen) pfad einer datei bestimmen.
	const std::string findFile(const std::string& filename) const;

private:

	//suchpfadeintrag. prioritt und pfad
	struct SearchPathEntry
	{
		//konstruktor fr einen searchpath, pfad(als string) und prioritt (als int)
		SearchPathEntry(std::string str, int i)
		{
			priority = i;
			path = str;
		}
		
		int priority;
		std::string path;

		//operator '<' berschreiben (um sortieren zu knnen)
		bool operator <(const SearchPathEntry& rhs) const
		{
			return priority < rhs.priority;
		}
	};

	//vector, der pfade speichert.
	std::vector<SearchPathEntry> searchPaths;

	//position eines pfades suchen
	bool findSearchPath(const std::string& str, std::vector<SearchPathEntry>::iterator& it); 

};

#endif
