/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mail‰nder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#ifndef FONT_H_INCLUDED
#define FONT_H_INCLUDED

#include "basicresourcemanager.h"
#include "resource.h"
#include "texture.h"

class Font;

// Ressourcenmanager für Fonts
BEGIN_IMPLEMENT_BASIC_RESOURCE_MANAGER(FontManager, Font)
END_IMPLEMENT_BASIC_RESOURCE_MANAGER

typedef Handle<Font> FontHandle;

class Font
	: public Resource
{
public:
	Font(FontManager& manager, const std::string& filename, const ParameterMap& options);

	// zeichnet einen Text und liefert dessen Größe zurück
	const Vec2f drawText(const Vec2f& pos, float size, const RGBA& color, const std::string& text);

	// zeichnet Text zentriert und liefert dessen Größe zurück
	const Vec2f drawCenteredText(const Vec2f& pos, float size, const RGBA& color, const std::string& text);

	// berechnet die Größe eines Texts
	const Vec2f calculateTextSize(float size, const std::string& text);

private:
	struct FontChar {
		int x;
		int y;
		int width;
		int height;
		int xoffset;
		int yoffset;
		int xadvance;
	};

	int lineHeight;
	int base;
	int scaleW;
	int scaleH;
	std::map<int, FontChar> chars;

	TextureHandle textureHandle;
};

#endif
