/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#ifndef GAMESTATE_H_INCLUDED
#define GAMESTATE_H_INCLUDED

#include "font.h"
#include "sound.h"
#include "state.h"
#include "texture.h"

class Level;
class PlayerObject;

// Spielmodi
enum GameMode
{
	GM_STORY,	// Storymodus (S)
	GM_TIME_1P,	// Zeitmodus mit einem Spieler (1)
	GM_TIME_2P,	// Zeitmodus mit zwei Spielern (2)
	GM_CTG,		// Capture the Guhaselnut (C)
	GM_KOH,		// King of the Hill (K)

	GM_MAX_VALUE
};

// Zustand fr das eigentliche Spiel
class GameState
	: public State
{
public:
	GameState(Application& app);

	void onEnter(const ParameterMap& params);

	void onLeave();

	void onGotFocus();

	void onLostFocus();

	void update();

	void render();

	// liefert den Spielmodus
	GameMode getGameMode() const;

private:
	// ldt einen Level (leerer Dateiname entldt nur den aktuellen Level)
	void loadLevel(const std::string& filename);

	// holt die Spielerobjekte
	void getPlayers();

	// rendert das HUD
	void renderHUD(int life = 75, int points = 1337);

	// rendert den Start-Count-Down
	void drawStartCountDown();

	ParameterMap gameParams;				// alle Spielparameter
	Level* p_level;							// aktueller Level
	std::string levelFilename;				// Dateiname des aktuellen Levels
	SoundInstanceHandle musicInstance;		// Instanz der Musik des Levels
	PlayerObject* p_player[2];				// Spielerobjekte
	Vec2f cameraPosition[2];				// Kamerapositionen
	float cameraDistance[2];				// Kameradistanzen
	bool paused;							// Spiel pausiert?
	TextureHandle hudOverlay;				// HUD-Overlay
	FontHandle hudFont;						// Schriftart fr das HUD
	FontHandle bigFont;						// groe Schriftart
	SoundHandle winnerSound;				// Fanfare fr den Gewinner
	SoundHandle loserSound;					// Fanfare fr den Verlierer

	int pointsNeededToWin;					// Punkte, die zum Gewinnen ntig sind (2-Spieler-Modus)
	int winner;								// Spieler, der gewonnen hat

	float time;								// Zeit seit Start des Levels
	float startCountDown;					// Start-Count-Down
	SoundHandle countDownSound;				// Count-Down-Sound
	SoundInstanceHandle countDownSoundInst;	// Instanz des Count-Down-Sounds
	float timeNeeded[2];					// bentigte Zeit der Spieler
	float bestTime;							// beste Zeit (oder -1, wenn es keine beste Zeit gibt)
	float timeToHighscores;					// Count-Down, bis es zu den Highscores geht
	bool cameFromHighscores;				// Kommen wir aus den Highscores?
};

#endif
