/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#ifndef HANDLE_H_INCLUDED
#define HANDLE_H_INCLUDED

// Template fr Handles ("Smart Pointer" fr referenzgezhlte Objekte)
template<typename T> class Handle
{
public:
	// Standardkonstruktor: kein Objekt zugewiesen
	Handle();

	// neu erzeugtes Objekt zuweisen
	explicit Handle(T* p_object);

	// Kopierkonstruktor
	Handle(const Handle& toCopy);

	// Kopierkonstruktor-Template mit Typumwandlung
	template<typename U> Handle(const Handle<U>& toCopy);

	// Destruktor: gibt das Objekt frei
	~Handle();

	// setzt den Handle zurck auf "kein Objekt"
	void reset();

	// liefert das Objekt des Handles
	T* get() const;

	// Casting-Operator nach bool
	operator bool() const;

	// Dereferenzierungsoperator
	T& operator *() const;

	// Pfeiloperator
	T* operator ->() const;

	// Kleiner-Operator
	bool operator <(const Handle<T>& rhs) const;

	// Zuweisungsoperator
	Handle& operator =(const Handle<T>& rhs);

private:
	T* p_object;
};

template<typename T> Handle<T>::Handle()
	: p_object(0)
{

}

template<typename T> Handle<T>::Handle(T* p_object)
	: p_object(p_object)
{

}

template<typename T> Handle<T>::Handle(const Handle& toCopy)
	: p_object(toCopy.p_object)
{
	if(p_object) p_object->addReference();
}

template<typename T> template<typename U> Handle<T>::Handle(const Handle<U>& toCopy)
	: p_object(toCopy.get())
{
	if(p_object) p_object->addReference();
}

template<typename T> Handle<T>::~Handle()
{
	reset();
}

template<typename T> void Handle<T>::reset()
{
	if(p_object)
	{
		p_object->releaseReference();
		p_object = 0;
	}
}

template<typename T> T* Handle<T>::get() const
{
	return p_object;
}

template<typename T> Handle<T>::operator bool() const
{
	return p_object != 0;
}

template<typename T> T& Handle<T>::operator *() const
{
	assert(p_object);
	return *p_object;
}

template<typename T> T* Handle<T>::operator ->() const
{
	assert(p_object);
	return p_object;
}

template<typename T> bool Handle<T>::operator <(const Handle& rhs) const
{
	return p_object < rhs.p_object;
}

template<typename T> Handle<T>& Handle<T>::operator =(const Handle& rhs)
{
	// erst erhhen, ...
	if(rhs.p_object) rhs.p_object->addReference();

	// ... dann verringern
	if(p_object) p_object->releaseReference();

	p_object = rhs.p_object;

	return *this;
}

#endif
