/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mail‰nder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#ifndef HIGHSCOREMANAGER_H_INCLUDED
#define HIGHSCOREMANAGER_H_INCLUDED

#include "application.h"

typedef unsigned int HighscoreValueType;

struct Score
{
	std::string name;
	HighscoreValueType value;
};

struct ScoreSorterDesc
{
	bool operator () (Score s1, Score s2)
	{
		if(s1.value == s2.value) return toLower(s1.name) < toLower(s2.name);
		return s1.value > s2.value;
	}
};

struct ScoreSorterAsc
{
	bool operator () (Score s1, Score s2)
	{
		if(s1.value == s2.value) return toLower(s1.name) < toLower(s2.name);
		return s1.value < s2.value;
	}
};

enum ScoreType
{
	ST_NULL,
	ST_TIME,
	ST_POINTS,
	
	ST_MAX_VALUE
};

typedef std::vector<Score> ScoreListType;

class HighscoreManager
{
public:
	HighscoreManager(Application& app);
	
	void add(std::string level, ScoreType scoreType, std::string name, HighscoreValueType value);
	std::vector<Score> get(std::string level, ScoreType scoreType);
	void save();
	bool test(std::string level, ScoreType scoreType, HighscoreValueType value);
	int getMaxEntries();
		
private:
	typedef std::map<ScoreType, ScoreListType> STListType;
	typedef std::map<std::string, STListType> ScoreTableType;
	Application& app;
	ScoreTableType scores;
	size_t maxEntries;
	void load();
};

#endif
