/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#include "pch.h"
#include "application.h"
#include "guhaselnutobject.h"
#include "homebaseobject.h"
#include "level.h"
#include "playerobject.h"

HomebaseObject::HomebaseObject(Layer& layer,
							   const TiXmlElement* p_xml,
							   const ParameterMap& gameParams)
	: Object(layer, p_xml, gameParams)
	, playerNo(-1)
	, size(0.0f, 0.0f)
{
	// Dieses Objekt soll nur im "Capture the Guhaselnut"-Modus existieren.
	if(getGameMode() != GM_CTG) kill();

	// Textur holen
	level.addResourceToCache(sprites = app.getTextureManager().getResourceHandle("sprites/area.png"));

	// Spielernummer lesen
	p_xml->QueryIntAttribute("playerNo", &playerNo);

	// Gre lesen
	if(const char* p_size = p_xml->Attribute("size")) size = fromString<Vec2f>(p_size);
	visibilityRadius = std::max(size.x, size.y);

	// Physik
	b2BodyDef bodyDef;
	b2Body& body = createBody(bodyDef);
	b2PolygonDef polygonDef;
	polygonDef.SetAsBox(size.x * 0.5f, size.y * 0.5f);
	polygonDef.density = 1.0f;
	polygonDef.isSensor = true;
	body.CreateShape(&polygonDef);
	body.SetMassFromShapes();

	// Initialisierung fertigstellen
	finishInit(p_xml);

	// Position merken
	originalPosition = getPosition();
}

void HomebaseObject::update()
{
	setPosition(originalPosition);
	getBody(0).SetLinearVelocity(b2Vec2(0.0f, 0.0f));
}

void HomebaseObject::render()
{
	app.setBlending(true);
	app.setTexturing(true);
	sprites->bind();

	RGBA theColor(level.getPlayer(playerNo)->getColor());
	theColor.a = 0.4f + 0.2f * sinf(20.0f * app.getTime());
	drawBodyQuad(0, Vec2i(0, 0), Vec2i(128, 128), size, false, false, theColor);
}

void HomebaseObject::onCollision(const Contact& contact)
{
	if(GuhaselnutObject* p_nut = dynamic_cast<GuhaselnutObject*>(contact.p_otherObject))
	{
		// die Nuss informieren
		p_nut->onCapture(*level.getPlayer(playerNo));
	}
}
