/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#include "pch.h"
#include "application.h"
#include "imagelayer.h"
#include "level.h"

ImageLayer::ImageLayer(Level& level,
					   int index,
					   const TiXmlElement* p_xml,
					   const ParameterMap& gameParams)
	: Layer(level, index, p_xml, gameParams)
	, imageOffset(0.0f, 0.0f)
	, tiling(1.0f, 1.0f)
	, movement(0.0f, 0.0f)
	, position(0.0f, 0.0f)
{
	// Dateiname des Bilds holen und das Bild laden
	const std::string imageFilename(p_xml->Attribute("imageFilename"));
	image = app.getTextureManager().getResourceHandle("sprites/" + imageFilename);

	// Offset des Bilds lesen
	if(const char* p_imageOffset = p_xml->Attribute("imageOffset")) imageOffset = fromString<Vec2f>(p_imageOffset);

	// Tiling lesen
	if(const char* p_tiling = p_xml->Attribute("tiling")) tiling = fromString<Vec2f>(p_tiling);

	// Bewegungsvektor lesen
	if(const char* p_movement = p_xml->Attribute("movement")) movement = fromString<Vec2f>(p_movement);
}

void ImageLayer::update()
{
	updateObjects();

	position += app.getUpdateStepSize() * movement;
}

void ImageLayer::render(Object* p_forceRenderingOnTop)
{
	const Vec2f imageSize(image->getSize());
	const Vec2f levelSize(level.getSize());

	// Textur, Blending und Transformation setzen
	app.setTexturing(true);
	image->bind();
	setupBlending();
	glPushMatrix();
	setupMatrix();

	// Layer rendern
	glBegin(GL_QUADS);
	glColor4fv(color);
	const Vec2f texCoordOffset(-imageOffset - position);
	glTexCoord2f(texCoordOffset.x, texCoordOffset.y);
	glVertex2f(0.0f, 0.0f);
	glTexCoord2f(tiling.x * imageSize.x + texCoordOffset.x, texCoordOffset.y);
	glVertex2f(levelSize.x, 0.0f);
	glTexCoord2f(tiling.x * imageSize.x + texCoordOffset.x, tiling.y * imageSize.y + texCoordOffset.y);
	glVertex2f(levelSize.x, levelSize.y);
	glTexCoord2f(texCoordOffset.x, tiling.y * imageSize.y + texCoordOffset.y);
	glVertex2f(0.0f, levelSize.y);
	glEnd();

	// Objekte rendern
	renderObjects(p_forceRenderingOnTop);

	glPopMatrix();
}

void ImageLayer::createShapes(b2Body& body)
{

}
