/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#ifndef LAYER_H_INCLUDED
#define LAYER_H_INCLUDED

class Application;
class Level;
class Object;

// Klasse fr Layer eines Levels
class Layer
	: NonCopyable
{
public:
	// Konstruktor: initialisiert den Layer
	Layer(Level& level, int index, const TiXmlElement* p_xml, const ParameterMap& gameParams);

	// virtueller Destruktor
	virtual ~Layer();

	// aktualisiert den Layer
	virtual void update() = 0;

	// rendert den Layer
	virtual void render(Object* p_forceRenderingOnTop) = 0;

	// erzeugt die Shapes dieses Layers
	virtual void createShapes(b2Body& body) = 0;

	// fgt ein Objekt zum Layer hinzu
	void addObject(Object& object);

	// entfernt ein Objekt aus dem Layer
	void removeObject(Object& object);

	// entfernt ein Objekt aus dem Layer. Wenn alsoDelete auf false steht, wird das Objekt dabei nicht zerstrt
	void removeObject(Object& object, bool alsoDelete);

	// liefert ein Objekt anhand seiner ID
	Object* getObjectByID(const std::string& id) const;

	// liefert das Level-Objekt
	Level& getLevel() const;

	// liefert die ID des Layers
	const std::string& getID() const;

	// liefert den Index des Layers
	int getIndex() const;

	// liefert den z-Wert des Layers
	float getZ() const;

	// liefert die Farbe des Layers
	const RGBA& getColor() const;

	// liefert den Blending-Modus des Layers
	int getBlendMode() const;

	// liefert die Sichtbarkeit des Layers
	bool isVisible() const;

protected:
	typedef struct { Object* p_obj; bool alsoDelete; } ObjectListEntryType;
	typedef std::list<ObjectListEntryType> ObjectListType;
	typedef std::map<std::string, Object*> ObjectMapType;

	// setzt die Transformationsmatrix fr den Layer
	void setupMatrix() const;

	// setzt das Blending fr den Layer
	void setupBlending() const;

	// aktualisiert die Objekte auf dem Layer
	void updateObjects();

	// rendert die Objekte auf dem Layer
	void renderObjects(Object* p_forceRenderingOnTop);

	Level& level;					// der Level
	Application& app;				// das Anwendungsobjekt
	std::string id;					// ID des Layers
	int index;						// Index des Layers
	float z;						// z-Wert des Layers (Tiefe)
	RGBA color;						// Farbe des Layers
	int blendMode;					// Blending-Modus
	bool visible;					// Ist der Layer sichtbar?
	ObjectMapType objects;			// Objekte des Layers
	bool updatingObjects;			// Werden die Objekte gerade aktualisiert?
	ObjectListType objectsToAdd;	// Objekte, die hinzugefgt werden
	ObjectListType objectsToRemove;	// Objekte, die gelscht werden
};

#endif
