/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#ifndef LEVEL_H_INCLUDED
#define LEVEL_H_INCLUDED

#include "sound.h"

class Layer;
class Object;
class PlayerObject;

// Klasse fr einen Level
class Level
	: NonCopyable
	, public b2ContactListener
{
public:
	// Konstruktor: initialisiert den Level
	Level(Application& app, const TiXmlElement* p_xml, const ParameterMap& gameParams);

	// Destruktor: zerstrt den Level
	~Level();

	// aktualisiert den Level
	void update();

	// rendert den Level
	void render(Object* p_forceRenderingOnTop = 0);

	// fgt eine Ressource zum Ressourcen-Cache hinzu
	void addResourceToCache(ResourceHandle resource);

	// liefert das Anwendungsobjekt
	Application& getApplication() const;

	// liefert die Gre des Levels
	const Vec2i& getSize() const;

	// liefert die Hintergrundmusik
	const std::string& getMusicFilename() const;

	// liefert einen Layer anhand seiner ID
	Layer* getLayerByID(const std::string& id) const;

	// liefert ein Objekt anhand seiner ID
	Object* getObjectByID(const std::string& id) const;

	// liefert ein Spielerobjekt
	PlayerObject* getPlayer(int playerNo) const;

	// liefert die Position der Kamera
	const Vec2f& getCameraPosition() const;

	// setzt die Kameraposition
	void setCameraPosition(const Vec2f& cameraPosition);

	// liefert die Kameraentfernung
	float getCameraDistance() const;

	// setzt die Kameraentfernung
	void setCameraDistance(float cameraDistance);

	// liefert die Physik-Welt
	b2World& getWorld() const;

	//////////////////////////////////////////////////////////////////////////

	void Add(const b2ContactPoint* p_point);

	void Persist(const b2ContactPoint* p_point);

private:
	typedef std::list<Layer*> LayerListType;
	typedef std::list<ResourceHandle> ResourceHandleListType;

	// fgt einen Layer hinzu
	void addLayer(Layer& layer);

	// korrigiert die Kamera (Position und Entfernung)
	void correctCamera();

	Application& app;					// das Anwendungsobjekt
	Vec2i size;							// Gre des Levels (in Tiles)
	std::string musicFilename;			// Dateiname der Musik
	RGBA backgroundColor;				// Hintergrundfarbe
	bool useBackgroundColor;			// Hintergrundfarbe benutzen?
	LayerListType layers;				// Layer des Levels
	ResourceHandleListType resources;	// Cache fr Ressourcen
	Vec2f cameraPosition;				// Position der Kamera
	float cameraDistance;				// Entfernung der Kamera vom Spielfeld
	b2World* p_world;					// Physik-Welt
	Vec2f gravity;						// Gravitation
};

#endif
