/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#include "pch.h"
#include "application.h"
#include "gamestate.h"
#include "mainmenustate.h"
#include "physicsstate.h"
#include "creditsstate.h"
#include "highscorestate.h"

int main(int numArgs,
		 char** pp_args)
{
#ifdef __MACOSX__
	const std::string dataPath("Rollmorad.app/Contents/Resources");
#else
	const std::string dataPath("data");
#endif

#ifdef _DEBUG
	const bool fullScreen = false;
#else
	const bool fullScreen = true;
#endif

	// Anwendung initialisieren
	Application app(Vec2i(1024, 768), fullScreen, 60.0f, "Rollmorad Guhase 2010", dataPath + "/sprites/icon.png");
	app.setCursor(false);

	// Pfade fr Daten registrieren
	app.getFileLocator().addSearchPath(dataPath);

	// Screenshot-Sound setzen
	app.setScreenshotSound(app.getSoundManager().getResourceHandle("sounds/camera.ogg"));

	// Zustnde registrieren
	// (die Anwendung bernimmt die Verantwortung fr die Objekte)
	app.registerState(new MainMenuState(app));
	app.registerState(new GameState(app));
	app.registerState(new PhysicsState(app));
	app.registerState(new CreditsState(app));
	app.registerState(new HighscoreState(app));

	// Startzustand auf den Stack legen
	app.pushStateByName("MainMenu");

	// Anwendung starten
	app.run();

	return 0;
}
