/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#include "pch.h"
#include "application.h"
#include "highscoremanager.h"
#include "mainmenustate.h"
#include "transitions.h"

MainMenuState::MainMenuState(Application& app)
	: State("MainMenu", app)
	, selectedEntry(ME_LEVEL)
	, selectedMode(GM_TIME_1P)
	, selectedLevel(0)
	, numLevels(0)
	, pointsToWin(5)
	, logoBrightness(1.0f)
{
	// Schriftart und Sounds laden
	font = app.getFontManager().getResourceHandle("fonts/arial.fnt");
	gh = app.getSoundManager().getResourceHandle("sounds/gh.ogg");
	ghear = app.getSoundManager().getResourceHandle("sounds/ghear.ogg");
	gha = app.getSoundManager().getResourceHandle("sounds/gha.ogg");
	errorSound = app.getSoundManager().getResourceHandle("sounds/error.ogg");
	neon = app.getSoundManager().getResourceHandle("sounds/neon.ogg");

	for(int i = 0; i < ME_MAX_VALUE; ++i) menuEntryOffset[i] = 0.0f;
	controlModes[0] = CM_WILLI;
	controlModes[1] = CM_WILLI;
}

void MainMenuState::onEnter(const ParameterMap& params)
{
	logo = app.getTextureManager().getResourceHandle("sprites/logo.png");
	loadLevelList();
}

void MainMenuState::onLeave()
{

}

void MainMenuState::onGotFocus()
{
	// Musik abspielen
	app.setMusic("sounds/mainmenu.ogg", 0.6f);
	neonInst = neon->createInstance(true, true, 0.0f);
}

void MainMenuState::onLostFocus()
{
	neonInst->stop();
}

void MainMenuState::update()
{
	// Eingabe verarbeiten
	SDL_keysym key;
	while(app.hasMoreKeys())
	{
		key = app.popKey();
		switch (key.sym)
		{
		case SDLK_BACKSPACE:
			input = input.substr(0, input.length() - 1);
			break;
		case SDLK_RETURN:
			input += '\n';
			break;
		default:
			input += char(key.unicode);
			break;
		}
	}

	if(app.wasKeyPressed(SDLK_ESCAPE))
	{
		// Zustand verlassen
		app.popState();
		return;
	}

	if(app.wasKeyPressed(SDLK_F5))
	{
		// Physikzustand auf den Stack legen
		app.pushStateByName("Physics");
		app.startTransition(new CubeTransition(app, 0.75f));
	}

	if(app.wasKeyPressed(SDLK_DOWN)) selectedEntry = static_cast<MenuEntry>((selectedEntry + 1) % ME_MAX_VALUE), ghear->createInstance();
	if(app.wasKeyPressed(SDLK_UP)) selectedEntry = static_cast<MenuEntry>((ME_MAX_VALUE + (selectedEntry - 1)) % ME_MAX_VALUE), ghear->createInstance();

	for(int i = 0; i < ME_MAX_VALUE; ++i)
	{
		const float targetOffset = selectedEntry == static_cast<MenuEntry>(i) ? 0.75f : 0.0f;
		menuEntryOffset[i] = 0.8f * menuEntryOffset[i] + 0.2f * targetOffset;
	}

	ScoreType scoreType = ST_NULL;
	bool startGame = false;
	
	switch (selectedMode) {
	case GM_CTG:
		gameMode = "Capture the GUHASElnut";
		gameModeID = "GM_CTG";
		scoreType = ST_NULL;
		break;
	case GM_KOH:
		gameMode = "King of the Hill";
		gameModeID = "GM_KOH";
		scoreType = ST_NULL;
		break;
	case GM_STORY:
		gameMode = "Story";
		gameModeID = "GM_STORY";
		scoreType = ST_POINTS;
		break;
	case GM_TIME_1P:
		gameMode = "Zeit, 1 Spieler";
		gameModeID = "GM_TIME_1P";
		scoreType = ST_TIME;
		break;
	case GM_TIME_2P:
		gameMode = "Zeit, 2 Spieler";
		gameModeID = "GM_TIME_2P";
		scoreType = ST_TIME;
		break;
	}
	
	ParameterMap params;

	switch (selectedEntry) {
		case ME_LEVEL:
			if(app.wasKeyPressed(SDLK_RIGHT)) selectedLevel = (selectedLevel + 1) % numLevels, gh->createInstance();
			if(app.wasKeyPressed(SDLK_LEFT)) selectedLevel = (selectedLevel + numLevels - 1) % numLevels, gh->createInstance();
			if(app.wasKeyPressed(SDLK_RETURN)) startGame = true;
			break;
		case ME_GAME_MODE:
			if(app.wasKeyPressed(SDLK_RIGHT))
			{
				selectedMode = static_cast<GameMode>((selectedMode + 1) % GM_MAX_VALUE), gh->createInstance();
				if(selectedMode == GM_STORY) selectedMode = static_cast<GameMode>((selectedMode + 1) % GM_MAX_VALUE);
			}
			if(app.wasKeyPressed(SDLK_LEFT))
			{
				selectedMode = static_cast<GameMode>((GM_MAX_VALUE + (selectedMode - 1)) % GM_MAX_VALUE), gh->createInstance();
				if(selectedMode == GM_STORY) selectedMode = static_cast<GameMode>((selectedMode - 1) % GM_MAX_VALUE);
			}
			if(app.wasKeyPressed(SDLK_RETURN)) startGame = true;
			break;
		case ME_POINTS:
			if(app.wasKeyPressed(SDLK_RIGHT)) ++pointsToWin, gh->createInstance();
			if(app.wasKeyPressed(SDLK_LEFT)) --pointsToWin, gh->createInstance();
			pointsToWin = std::max(1, std::min(200, pointsToWin));
			if(app.wasKeyPressed(SDLK_RETURN)) startGame = true;
			break;
		case ME_CONTROL_MODE:
			if(app.wasKeyPressed(SDLK_RIGHT))
			{
				controlModes[0] = static_cast<ControlMode>((controlModes[0] + 1) % CM_MAX_VALUE);
				if(!controlModes[0]) controlModes[1] = static_cast<ControlMode>((controlModes[1] + 1) % CM_MAX_VALUE);
				gh->createInstance();
			}
			if(app.wasKeyPressed(SDLK_LEFT))
			{
				controlModes[0] = static_cast<ControlMode>((controlModes[0] + CM_MAX_VALUE - 1) % CM_MAX_VALUE);
				if(controlModes[0] == CM_MAX_VALUE - 1) controlModes[1] = static_cast<ControlMode>((controlModes[1] + CM_MAX_VALUE - 1) % CM_MAX_VALUE);
				gh->createInstance();
			}
			if(app.wasKeyPressed(SDLK_RETURN)) startGame = true;
			break;
		case ME_START_GAME:
			if(app.wasKeyPressed(SDLK_RETURN)) startGame = true;
			break;
		case ME_CREDITS:
			if(app.wasKeyPressed(SDLK_RETURN))
			{
				gha->createInstance();
				app.pushStateByName("Credits");
				app.startTransition(new CubeTransition(app, 0.75f));
				return;
			}
			break;
		case ME_HIGHSCORE:
			if(app.wasKeyPressed(SDLK_RETURN))
			{
				// Highscores checken
				HighscoreManager highscoreMgr(app);
				if(highscoreMgr.get(levelFilenames[selectedLevel], scoreType).empty())
				{
					// keine Highscores
					errorSound->createInstance();
				}
				else
				{
					gha->createInstance();
					params.set<ScoreType>("scoreType", scoreType);
					params.set<std::string>("levelFilename", levelFilenames[selectedLevel]);

					// die folgenden beiden Parameter weglassen, um den Highscore nur anzuzeigen
//					params.set<std::string>("name1", "Herbert");
//					params.set<unsigned int>("score1", 170);

					app.pushStateByName("Highscore", params);
					app.startTransition(new CubeTransition(app, 0.75f));
				}
				return;
			}
			break;
		case ME_EXIT:
			if(app.wasKeyPressed(SDLK_RETURN))
			{
				app.popState();
				return;
			}
			break;
	}
	
	if(startGame)
	{
		if(levelGameModes[selectedLevel].find(gameModeID) == -1) errorSound->createInstance();
		else
		{
			gha->createInstance();
			params.set<GameMode>("gameMode", selectedMode);
			params.set<int>("pointsNeededToWin", pointsToWin);
			params.set<std::string>("levelFilename", levelFilenames[selectedLevel]);
			params.set<ControlMode>("controlMode1", controlModes[0]);
			params.set<ControlMode>("controlMode2", controlModes[1]);
			app.pushStateByName("Game", params);
			app.startTransition(new CubeTransition(app, 0.75f));
			return;
		}
	}

	// Logo flimmern lassen
	if(!MTRand::globalMTRand.randomS32(0, 1000)) logoBrightness = MTRand::globalMTRand.randomFloat(-20.0f, 0.0f);
	logoBrightness = 0.75f * logoBrightness + 0.25f;
	logoBrightness += MTRand::globalMTRand.randomFloat(-0.1f, 0.1f) + 0.05f * sinf(app.getTime());
	neonInst->setVolume((1.0f - logoBrightness) / 1.5f);
}

void MainMenuState::render()
{
	static const RGBA normalCol(0.15f, 0.65f, 0.82f);
	static const RGBA highlightCol(1.0f, 1.0f, 1.0f);

	app.clearViewport(RGBA::BLACK);

	// Projektionsmatrix einstellen
	glMatrixMode(GL_PROJECTION);
	glOrtho(0.0, 32.0, 24.0, 0.0, 0.0, 1.0);

	glMatrixMode(GL_MODELVIEW);

	float yPos = 0.5f;
	float xPos = 19.5f;

	// Logo
	const int logoWidth = 395;
	const int logoHeight = 653;
	const float logoAspect = float(logoHeight) / float(logoWidth);
	const float logoScale = 12.0f;

	app.setBlending(true, GL_ONE, GL_ONE);
	app.setTexturing(true);
	logo->bind();
	glBegin(GL_QUADS);
		glColor4fv(RGBA::WHITE);
		glTexCoord2i(0, 0);
		glVertex2f(xPos, yPos);
		glTexCoord2i(logoWidth, 0);
		glVertex2f(xPos + logoScale, yPos);
		glTexCoord2i(logoWidth, logoHeight);
		glVertex2f(xPos + logoScale, yPos + (logoScale * logoAspect));
		glTexCoord2i(0, logoHeight);
		glVertex2f(xPos, yPos + (logoScale * logoAspect));
		glColor4fv(RGBA(logoBrightness));
		glTexCoord2i(512, 0);
		glVertex2f(xPos, yPos);
		glTexCoord2i(512 + logoWidth, 0);
		glVertex2f(xPos + logoScale, yPos);
		glTexCoord2i(512 + logoWidth, logoHeight);
		glVertex2f(xPos + logoScale, yPos + (logoScale * logoAspect));
		glTexCoord2i(512, logoHeight);
		glVertex2f(xPos, yPos + (logoScale * logoAspect));
	glEnd();
	
	// Men
	xPos = 1.0f;
	yPos = 5.5f;
	const float ySpace = 2.0f;
	const float separator = 1.0f;
	RGBA color;

	color = selectedEntry == ME_LEVEL ? highlightCol : normalCol;
	font->drawText(Vec2f(xPos + menuEntryOffset[ME_LEVEL],  yPos), 1.0f, color, std::string("Level:   ") + levelNames[selectedLevel]);
	yPos += ySpace;

	color = selectedEntry == ME_GAME_MODE ? highlightCol : normalCol;
	font->drawText(Vec2f(xPos + menuEntryOffset[ME_GAME_MODE], yPos), 1.0, color, std::string("Spielmodus:   ") + gameMode);
	yPos += ySpace;

	color = selectedEntry == ME_POINTS ? highlightCol : normalCol;
	font->drawText(Vec2f(xPos + menuEntryOffset[ME_POINTS], yPos), 1.0f, color, std::string("Zielpunktzahl:   ") + toString(pointsToWin));
	yPos += ySpace;

	color = selectedEntry == ME_CONTROL_MODE ? highlightCol : normalCol;
	static const char* p_controlModeNames[CM_MAX_VALUE] = { "5 Tasten", "4 Tasten" };
	font->drawText(Vec2f(xPos + menuEntryOffset[ME_CONTROL_MODE], yPos), 1.0f, color, std::string("Spielersteuerung 1 / 2:   ") + p_controlModeNames[controlModes[0]] + " / " + p_controlModeNames[controlModes[1]]);
	yPos += ySpace;

	color = selectedEntry == ME_START_GAME ? highlightCol : normalCol;
	font->drawText(Vec2f(xPos + menuEntryOffset[ME_START_GAME], yPos), 1.0f, color, "Spiel starten");
	yPos += ySpace + separator;

	color = selectedEntry == ME_HIGHSCORE ? highlightCol : normalCol;
	font->drawText(Vec2f(xPos + menuEntryOffset[ME_HIGHSCORE], yPos), 1.0f, color, "Highscore");
	yPos += ySpace;
	
	color = selectedEntry == ME_CREDITS ? highlightCol : normalCol;
	font->drawText(Vec2f(xPos + menuEntryOffset[ME_CREDITS], yPos), 1.0f, color, "Credits");
	yPos += ySpace + separator;
		
	color = selectedEntry == ME_EXIT ? highlightCol : normalCol;
	font->drawText(Vec2f(xPos + menuEntryOffset[ME_EXIT], yPos), 1.0f, color, "Beenden");
	yPos += ySpace;

	color = RGBA(1.0f, 0.0f, 1.0f);
	font->drawText(Vec2f(1.0f, 1.0f), 1.0f, color, input);
}

void MainMenuState::loadLevelList()
{
	levelFilenames.clear();
	levelNames.clear();

	// levels.txt ffnen
	std::ifstream str(app.getFileLocator().findFile("levels/levels.txt").c_str());
	while(str.good())
	{
		// lesen
		std::string levelFilename;
		str >> levelFilename;
		if(!levelFilename.empty())
		{
			// Level ffnen
			TiXmlDocument doc;
			if(doc.LoadFile(app.getFileLocator().findFile(std::string("levels/") + levelFilename).c_str()))
			{
				TiXmlElement* p_levelXML = doc.FirstChildElement("Level");
				if(p_levelXML)
				{
					// wenn kein Name angegeben ist, den Dateinamen nehmen
					const char* p_levelName = p_levelXML->Attribute("name");
					if(!p_levelName) p_levelName = levelFilename.c_str();

					// untersttzte Spielmodi lesen
					const char* p_supportedGameModes = p_levelXML->Attribute("supportedGameModes");
					if(!p_supportedGameModes) p_supportedGameModes = "";

					// eintragen
					levelFilenames.push_back(levelFilename);
					levelNames.push_back(p_levelName);
					levelGameModes.push_back(p_supportedGameModes);
				}
			}
		}
	}

	numLevels = levelFilenames.size();
}
