/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#include "pch.h"

MTRand MTRand::globalMTRand;

MTRand::MTRand(Uint32 seed)
{
	this->seed(seed);
}

MTRand::MTRand(const MTRand& toCopy)
{
	*this = toCopy;
}

void MTRand::seed(Uint32 seed)
{
	next = 0;
	const Uint64 factor = 69069;

	mt[0] = seed;
	for(unsigned int i = 1; i < 624; ++i)
	{
		mt[i] = static_cast<Uint32>(factor * mt[i - 1] + 1);
	}
}

Uint32 MTRand::randomU32()
{
	if(next == 624)
	{
		generateNumbers();
		next = 0;
	}

	Uint32 x = mt[next++];
	x ^= x >> 11;
	x ^= (x << 7) & 0x9D2C5680;
	x ^= (x << 15) & 0xEFC60000;
	x ^= x >> 18;
	return x;
}

Uint64 MTRand::randomU64()
{
	return (static_cast<Uint64>(randomU32()) << 32) | randomU32();
}

float MTRand::randomFloat01()
{
	return static_cast<float>(randomU32() & 0xFFFFFF) / static_cast<float>(0xFFFFFF);
}

float MTRand::randomFloat01Exclusive()
{
	union { unsigned int i; float f; } value = { 0x3F800000 | (randomU32() >> 9) };
	return value.f - 1.0f;
}

double MTRand::randomDouble01()
{
	return static_cast<double>(randomU64() & 0x1FFFFFFFFFFFFFULL) / static_cast<double>(0x1FFFFFFFFFFFFFULL);
}

double MTRand::randomDouble01Exclusive()
{
	union { unsigned long long int i; double f; } value = { 0x3FF0000000000000ULL | (randomU64() >> 12) };
	return value.f - 1.0;
}

Sint32 MTRand::randomS32(Sint32 lower,
						 Sint32 upper)
{
	const double r = randomDouble01Exclusive();
	return lower + static_cast<Sint32>(r * (upper - lower + 1));
}

Sint32 MTRand::randomS32Exclusive(Sint32 lower,
								  Sint32 upper)
{
	const double r = randomDouble01Exclusive();
	return lower + static_cast<Sint32>(r * (upper - lower));
}

float MTRand::randomFloat(float lower,
						  float upper)
{
	return lower + randomFloat01() * (upper - lower);
}

float MTRand::randomFloatExclusive(float lower,
								   float upper)
{
	return lower + randomFloat01Exclusive() * (upper - lower);
}

double MTRand::randomDouble(double lower,
							double upper)
{
	return lower + randomDouble01() * (upper - lower);
}

double MTRand::randomDoubleExclusive(double lower,
									 double upper)
{
	return lower + randomDouble01Exclusive() * (upper - lower);
}

MTRand& MTRand::operator =(const MTRand& rhs)
{
	for(unsigned int i = 0; i < 624; ++i) mt[i] = rhs.mt[i];
	next = rhs.next;
	return *this;
}

void MTRand::generateNumbers()
{
	const Uint32 xorTab[] = { 0, 0x9908B0DF };

	for(unsigned int i = 0; i < 227; ++i)
	{
		Uint32 x = (mt[i] & 0x80000000) | (mt[i + 1] & 0x7FFFFFFF);
		mt[i] = mt[i + 397] ^ (x >> 1) ^ xorTab[x & 1];
	}

	for(unsigned int i = 227; i < 624; ++i)
	{
		Uint32 x = (mt[i] & 0x80000000) | (mt[i + 1] & 0x7FFFFFFF);
		mt[i] = mt[i - 227] ^ (x >> 1) ^ xorTab[x & 1];
	}

	Uint32 x = (mt[623] & 0x80000000) | (mt[0] & 0x7FFFFFFF);
	mt[623] = mt[396] ^ (x >> 1) ^ xorTab[x & 1];
}
