/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#ifndef MTRAND_H_INCLUDED
#define MTRAND_H_INCLUDED

// Mersenne Twister-Zufallsgenerator
class MTRand
{
public:
	// Initialisierung mit Startwert
	MTRand(Uint32 seed = 31337);

	// Kopierkonstruktor
	MTRand(const MTRand& toCopy);

	// mit neuem Startwert initialisieren
	void seed(Uint32 seed);

	// liefert einen zuflligen 32-Bit-Integer ohne Vorzeichen
	Uint32 randomU32();

	// liefert einen zuflligen 64-Bit-Integer ohne Vorzeichen
	Uint64 randomU64();

	// liefert einen zuflligen 32-Bit-Fliekommawert in [0, 1]
	float randomFloat01();

	// liefert einen zuflligen 32-Bit-Fliekommawert in [0, 1)
	float randomFloat01Exclusive();

	// liefert einen zuflligen 64-Bit-Fliekommawert in [0, 1]
	double randomDouble01();

	// liefert einen zuflligen 64-Bit-Fliekommawert in [0, 1)
	double randomDouble01Exclusive();

	// liefert einen zuflligen 32-Bit-Integer in [lower, upper]
	Sint32 randomS32(Sint32 lower, Sint32 upper);

	// liefert einen zuflligen 32-Bit-Integer in [lower, upper)
	Sint32 randomS32Exclusive(Sint32 lower, Sint32 upper);

	// liefert einen zuflligen 32-Bit-Fliekommawert in [lower, upper]
	float randomFloat(float lower, float upper);

	// liefert einen zuflligen 32-Bit-Fliekommawert in [lower, upper)
	float randomFloatExclusive(float lower, float upper);

	// liefert einen zuflligen 64-Bit-Fliekommawert in [lower, upper]
	double randomDouble(double lower, double upper);

	// liefert einen zuflligen 64-Bit-Fliekommawert in [lower, upper)
	double randomDoubleExclusive(double lower, double upper);

	// Zuweisungsoperator
	MTRand& operator =(const MTRand& rhs);

	// "globaler" Zufallsgenerator
	static MTRand globalMTRand;

private:
	// generiert neue Zahlen
	void generateNumbers();

	Uint32 mt[624];
	Uint32 next;
};

#endif
