/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#include "pch.h"

ParameterMap::ParameterMap()
{

}

ParameterMap::ParameterMap(const ParameterMap& toCopy)
{
	*this = toCopy;
}

ParameterMap::~ParameterMap()
{
	clear();
}

bool ParameterMap::isEmpty() const
{
	return params.empty();
}

void ParameterMap::clear()
{
	// alle Parameter lschen
	for(ParameterMapType::const_iterator it = params.begin();
		it != params.end();
		++it)
	{
		delete it->second;
	}

	// leeren
	params.clear();
}

void ParameterMap::merge(const ParameterMap& other)
{
	// alle Parameter von der anderen ParameterMap herberkopieren
	for(ParameterMapType::const_iterator it = other.params.begin();
		it != other.params.end();
		++it)
	{
		// Schon vorhanden? Wenn ja, dann lschen.
		ParameterMapType::iterator jt = params.find(it->first);
		if(jt != params.end())
		{
			delete jt->second;
			params.erase(jt);
		}

		// Parameter klonen
		params[it->first] = it->second->clone();
	}
}

bool ParameterMap::has(const std::string& name) const
{
	// Eintrag vorhanden?
	return params.find(name) != params.end();
}

void ParameterMap::remove(const std::string& name)
{
	ParameterMapType::iterator it = params.find(name);
	delete it->second;
	params.erase(it);
}

const std::string ParameterMap::toString() const
{
	std::ostringstream stream;
	stream << '{';

	// alle Parameter serialisieren
	bool first = true;
	for(ParameterMapType::const_iterator it = params.begin();
		it != params.end();
		++it)
	{
		if(first)
		{
			stream << ", ";
			first = false;
		}

		stream << it->first << ": " << it->second->toString();
	}

	stream << '}';
	return stream.str();
}

bool ParameterMap::operator ==(const ParameterMap& rhs) const
{
	return toString() == rhs.toString();
}

bool ParameterMap::operator <(const ParameterMap& rhs) const
{
	return toString() < rhs.toString();
}

ParameterMap& ParameterMap::operator =(const ParameterMap& rhs)
{
	clear();

	for(ParameterMapType::const_iterator it = rhs.params.begin();
		it != rhs.params.end();
		++it)
	{
		// Parameter klonen
		params[it->first] = it->second->clone();
	}

	return *this;
}

std::ostream& operator <<(std::ostream& lhs,
						  const ParameterMap& rhs)
{
	return lhs << rhs.toString();
}
