/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#ifndef PARAMETERMAP_H_INCLUDED
#define PARAMETERMAP_H_INCLUDED

#include "basicparameter.h"

// Klasse zur bergabe beliebiger Parameter
class ParameterMap
{
public:
	// Standardkonstruktor: leere Parameter-Map
	ParameterMap();

	// Kopierkonstruktor
	ParameterMap(const ParameterMap& toCopy);

	// Destruktor: lscht alle Parameter
	~ParameterMap();

	// testet, ob die ParameterMap leer ist
	bool isEmpty() const;

	// leert die ParameterMap
	void clear();

	// fgt eine zweite ParameterMap ein (vorhandene Parameter werden berschrieben)
	void merge(const ParameterMap& other);

	// prft, ob ein Parameter vorhanden ist
	bool has(const std::string& name) const;

	// fragt einen Parameter ab, der auch existieren muss
	template<typename T> const T& get(const std::string& name) const;

	// fragt einen Parameter ab (muss nicht existieren)
	template<typename T> bool query(const std::string& name, T& outValue) const;

	// setzt einen Parameter (alter Wert, falls vorhanden, wird berschrieben)
	template<typename T> void set(const std::string& name, const T& value);

	// entfernt einen Parameter
	void remove(const std::string& name);

	// wandelt die ParameterMap in einen String um
	const std::string toString() const;

	// Vergleichsoperator
	bool operator ==(const ParameterMap& rhs) const;

	// Kleiner-Operator
	bool operator <(const ParameterMap& rhs) const;

	// Zuweisungsoperator
	ParameterMap& operator =(const ParameterMap& rhs);

private:
	typedef std::map<std::string, Parameter*> ParameterMapType;

	ParameterMapType params;
};

template<typename T> const T& ParameterMap::get(const std::string& name) const
{
	ParameterMapType::const_iterator it = params.find(name);
	assert(dynamic_cast<const BasicParameter<T>*>(it->second));
	return static_cast<const BasicParameter<T>*>(it->second)->getValue();
}

template<typename T> bool ParameterMap::query(const std::string& name,
											  T& outValue) const
{
	if(has(name))
	{
		// Der Wert existiert!
		outValue = get<T>(name);
		return true;
	}

	// Der Wert existiert nicht! Ist aber nicht schlimm.
	return false;
}

template<typename T> void ParameterMap::set(const std::string& name,
											const T& value)
{
	typename ParameterMapType::const_iterator it = params.find(name);
	if(it != params.end())
	{
		// alten Wert lschen
		delete it->second;
	}

	// neuen Wert eintragen
	params[name] = new BasicParameter<T>(value);
}

// Stream-Ausgabe einer ParameterMap
std::ostream& operator <<(std::ostream& lhs, const ParameterMap& rhs);

#endif
