/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#ifndef PHYSICSSTATE_H_INCLUDED
#define PHYSICSSTATE_H_INCLUDED

#include "sound.h"
#include "state.h"
#include "texture.h"
#include "font.h"

#define PHYSICSSTATE_MAXBODIES 400

// Zustand fr die Physik-Engine
class PhysicsState
	: public State
{
public:
	PhysicsState(Application& app);

	void onEnter(const ParameterMap& params);

	void onLeave();

	void onGotFocus();

	void onLostFocus();

	void update();

	void render();

private:
	FontHandle font;
	b2AABB worldAABB;
	b2World* world;
	b2Body* bodies[PHYSICSSTATE_MAXBODIES]; // Statt Spielobjekten werden hier direkt die Physik-Objekte gerendert
	int bodyIndex; // Position, an der neuer Body eingefgt wird
	int numBodies; // Gesamtanzahl an Bodies (Fllstand des bodies-Array)
	void drawBody(b2Body* body);
	void addBody(float x, float y);
	void removeBody();
	
	b2Body* body1;
	b2Body* body2;
	b2Body* body3;
	b2RevoluteJoint* rj;
};

#endif
