/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#ifndef PLAYEROBJECT_H_INCLUDED
#define PLAYEROBJECT_H_INCLUDED

#include "object.h"
#include "sound.h"
#include "texture.h"

// Steuerungsmodi
enum ControlMode
{
	CM_WILLI,
	CM_JAN,

	CM_MAX_VALUE
};

// Spielerobjekt
class PlayerObject
	: public Object
{
public:
	// Konstruktor: initialisiert das Objekt
	PlayerObject(Layer& layer, const TiXmlElement* p_xml, const ParameterMap& gameParams);

	// Destruktor
	~PlayerObject();

	void update();

	void render();

	void onCollision(const Contact& contact);
	
	int getPoints() const;

	void addPoints(int amount);

	void addPoints(float amount);

	void reduceHealth(int amount);

	void reduceHealth(float amount);

	int getHealth() const;

	void activate();

	void deactivate();

	const RGBA& getColor() const;

	bool isMirrored() const;

	void setMirrored(bool mirrored);

	void addVelocity(b2Vec2 velocity);
	
	void setLayerDelayed(std::string layerID);

private:
	enum Action
	{
		A_ACCELERATE,
		A_BRAKE,
		A_LEAN_FORWARD,
		A_LEAN_BACK,
		A_TURN,

		A_MAX_VALUE
	};
	
	enum Control
	{
		C_GO_FORWARD,
		C_GO_BACKWARD,
		C_LEAN_FORWARD,
		C_LEAN_BACKWARD,

		C_MAX_VALUE
	};

	// initialisiert die Physik
	void setup();

	TextureHandle sprites;
	int playerNo;
	RGBA color;
	int points;
	float pointsF;
	int health;
	float healthF;
	SoundHandle motorSound;
	SoundInstanceHandle motorSoundInstance;
	bool mirrored;
	float motorSpeed;
	Vec2f frontWheelCohesionForce;
	Vec2f backWheelCohesionForce;
	int frontWheelOnGround;
	int backWheelOnGround;
	bool active;
	ControlMode controlMode;
	bool changeLayer;
	std::string targetLayer;
};

#endif
