/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#ifndef RESOURCE_H_INCLUDED
#define RESOURCE_H_INCLUDED

#include "handle.h"

class Resource;
class ResourceManager;

// Handle fr Ressourcen
typedef Handle<Resource> ResourceHandle;

class Resource
	: NonCopyable
	, public ReferenceCounted
{
public:
	// Konstruktor
	Resource(ResourceManager& manager, const std::string& filename, const ParameterMap& options);

	// virtueller Destruktor: entfernt die Ressource aus dem Manager
	virtual ~Resource();

	// liefert den Ressourcenmanager
	ResourceManager& getManager() const;

	// liefert den Dateinamen der Ressource
	const std::string& getFilename() const;

	// liefert die Optionen, mit denen die Ressource geladen wurde
	const ParameterMap& getOptions() const;

	// liefert die Standardoptionen
	static const ParameterMap getDefaultOptions();

protected:
	ResourceManager& manager;
	const std::string filename;
	const ParameterMap options;
};

#endif
