/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#ifndef SOUND_H_INCLUDED
#define SOUND_H_INCLUDED

#include "basicresourcemanager.h"
#include "resource.h"
#include "soundinstance.h"

class Sound;

// Ressourcenmanager fr Sounds
BEGIN_IMPLEMENT_BASIC_RESOURCE_MANAGER(SoundManager, Sound)
	// aktualisiert alle Sounds
	void update();
END_IMPLEMENT_BASIC_RESOURCE_MANAGER

// Handle fr Sounds
typedef Handle<Sound> SoundHandle;

// Klasse fr Soundeffekte
class Sound
	: public Resource
{
public:
	// Konstruktor: ldt den Sound
	Sound(SoundManager& manager, const std::string& filename, const ParameterMap& options);

	// Destruktor: zerstrt den Sound
	~Sound();

	// erzeugt eine neue Instanz des Sounds
	SoundInstanceHandle createInstance(bool play = true, bool loop = false, float volume = 1.0f, float pitch = 1.0f);

	// aktualisiert den Sound (wird automatisch aufgerufen)
	void update();

	// liefert die Soundpuffer-ID (oder 0, wenn der Sound gestreamt wird)
	ALuint getSoundBufferID() const;

	// liefert true, wenn der Sound gestreamt werden soll
	bool getStreaming() const;

	// liefert das Format des Sounds
	ALenum getFormat() const;

	// liefert die Frequenz des Sounds
	ALsizei getFrequency() const;

	// liefert die Lnge des Sounds (in Sekunden)
	double getDuration() const;

	// liefert die Standardoptionen
	static const ParameterMap getDefaultOptions();

private:
	typedef std::list<SoundInstanceHandle> SoundInstanceListType;

	ALuint soundBufferID;				// OpenAL-Soundpuffer-ID (nur bei Sounds ohne Streaming)
	ALenum format;						// Format des Soundpuffers
	ALsizei frequency;					// native Abspielfrequenz (in Hz)
	double duration;					// Lnge des Sounds (in Sekunden)
	SoundInstanceListType instances;	// Liste der Soundinstanzen
};

#endif
