/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#ifndef STATE_H_INCLUDED
#define STATE_H_INCLUDED

#include <string>

class Application;
class ParameterMap;

class State
	: NonCopyable
{
public:
	// Konstruktor
	State(const std::string& name, Application& app);

	// virtueller Destruktor
	virtual ~State() { }

	// wird aufgerufen, wenn der Zustand betreten wird
	virtual void onEnter(const ParameterMap& params) { }

	// wird aufgerufen, wenn der Zustand verlassen wird
	virtual void onLeave() { }

	// wird aufgerufen, wenn der Zustand aktiviert wird
	virtual void onGotFocus() { }

	// wird aufgerufen, wenn der Zustand deaktiviert wird
	virtual void onLostFocus() { }

	// wird aufgerufen, wenn der Zustand sich aktualisieren soll
	virtual void update() { }

	// wird aufgerufen, wenn der Zustand sich rendern soll
	virtual void render() { }

	// liefert den Namen des Zustands
	const std::string& getName() const;

	// liefert das Anwendungsobjekt
	Application& getApplication() const;

protected:
	const std::string name;
	Application& app;
};

#endif
