/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#include "pch.h"
#include "application.h"
#include "level.h"
#include "textobject.h"

TextObject::TextObject(Layer& layer,
					   const TiXmlElement* p_xml,
					   const ParameterMap& gameParams)
	: Object(layer, p_xml, gameParams)
	, size(1.0f)
	, color(RGBA::WHITE)
	, velocity(0.0f, 0.0f)
	, timeToLive(0.0f)
	, age(0.0f)
	, fadeOut(true)
{
	// Text holen
	text = p_xml->Attribute("text");

	// Gre holen
	p_xml->QueryFloatAttribute("size", &size);

	// Farbe holen
	if(const char* p_color = p_xml->Attribute("color")) color = fromString<RGBA>(p_color);

	// Geschwindigkeit holen
	if(const char* p_velocity = p_xml->Attribute("velocity")) velocity = fromString<Vec2f>(p_velocity);

	// Lebensdauer holen
	p_xml->QueryFloatAttribute("timeToLive", &timeToLive);

	// Ausblendmodus holen
	int fadeOut = 1;
	p_xml->QueryIntAttribute("fadeOut", &fadeOut);
	this->fadeOut = fadeOut != 0;

	// Schriftart laden
	font = app.getFontManager().getResourceHandle("fonts/bellgothic.fnt");
	level.addResourceToCache(font);

	// Gre berechnen
	const Vec2f textSize(0.5f * font->calculateTextSize(size, text));
	visibilityRadius = std::max(textSize.x, textSize.y);

	// Initialisierung fertigstellen
	finishInit(p_xml);
}

void TextObject::update()
{
	// altern
	age += app.getUpdateStepSize();
	if(timeToLive > 0.0f && age >= timeToLive) kill();

	// bewegen
	position += app.getUpdateStepSize() * velocity;
}

void TextObject::render()
{
	glPushMatrix();

	const Vec2f position(getPosition());
	glTranslatef(position.x, position.y, 0.0f);
	glRotatef(rad2Deg(getAngle()), 0.0f, 0.0f, 1.0f);

	float theSize = size;
	RGBA theColor(color);
	if(timeToLive > 0.0f && fadeOut)
	{
		const float relativeAge = age / timeToLive;
		theSize = relativeAge * 3.0f * size + (1.0f - relativeAge) * size;
		theColor.a *= 1.0f - relativeAge;
	}

	font->drawCenteredText(Vec2f(0.0f, 0.0f), theSize, theColor, text);

	glPopMatrix();
}
