/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#ifndef TILE_H_INCLUDED
#define TILE_H_INCLUDED

#include "level.h"

// Klasse fr Informationen ber Tiles
class Tile
{
public:
	// Konstruktor: initialisiert das Tile
	Tile(const TiXmlElement* p_xml, const Vec2i& defaultSize, float defaultShapeRestitution, float defaultShapeFriction);

	// Destruktor
	~Tile();

	// zeichnet das Tile an der angegebenen Position (Textur ist bereits gesetzt)
	void draw(const Vec2f& position, const RGBA& color) const;

	// erzeugt die Shapes fr dieses Tile
	void createShapes(b2Body& body, Layer& layer, const Vec2f& position, const Vec2i& blockSize) const;

	// liefert die ID des Tiles
	int getID() const;

	// liefert true, wenn das Tile nur eine einfache 1x1-Box ist
	bool getIsSimple1x1Box() const;

	// liefert die Sichtbarkeit des Tiles
	bool isVisible() const;

private:
	typedef std::list<b2ShapeDef*> ShapeDefListType;

	int id;						// ID des Tiles
	Vec2i position;				// Texturkoordinaten der linken oberen Ecke
	Vec2i size;					// Gre auf der Textur
	bool isSimple1x1Box;		// Ist dieses Tile nur eine einfache 1x1-Box?
	ShapeDefListType shapeDefs;	// Shape-Definitionen
	bool visible;				// Sichtbar?
};

#endif
