/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#ifndef TILEMAPLAYER_H_INCLUDED
#define TILEMAPLAYER_H_INCLUDED

#include "layer.h"

class Tile;
class TileSet;

// Layer, der eine Tile-Map beinhaltet
class TileMapLayer
	: public Layer
{
public:
	// Konstruktor: initialisiert den Layer
	TileMapLayer(Level& level, int index, const TiXmlElement* p_xml, const ParameterMap& gameParams);

	// Destruktor: lscht den Layer
	~TileMapLayer();

	void update();

	void render(Object* p_forceRenderingOnTop);

	void createShapes(b2Body& body);

	// liefert die Tile-ID an einer bestimmten Stelle
	int getTileIDAt(const Vec2i& position) const;

	// liefert das Tile an einer bestimmten Stelle
	const Tile* getTileAt(const Vec2i& position) const;

private:
	void extendBlock(const Vec2i& position, Vec2i& blockSize, int& axes) const;

	TileSet* p_tileSet;	// das Tile-Set
	Vec2i size;			// Gre des Layers (in Tiles)
	Uint32* p_tiles;	// die Tiles (zeilenweise)
	bool enablePhysics;	// Physik fr diesen Layer?
};

#endif
