/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#ifndef UTILS_H_INCLUDED
#define UTILS_H_INCLUDED

template<typename T> struct Vec2;

// Pi als 32-Bit-Fliekommazahl
extern const float M_PI_FLOAT;

// Pi als 64-Bit-Fliekommazahl
extern const double M_PI_DOUBLE;

// 2 * Pi als 32-Bit-Fliekommazahl
extern const float M_2_PI_FLOAT;

// 2 * Pi als 64-Bit-Fliekommazahl
extern const double M_2_PI_DOUBLE;

// Umrechnung Grad -> Bogenma
inline float deg2Rad(float x)
{
	return x * M_PI_FLOAT / 180.0f;
}

// Umrechnung Bogenma -> Grad
inline float rad2Deg(float x)
{
	return x * 180.0f / M_PI_FLOAT;
}

// Filtertypen fr Herunterskalieren eines Bildes
enum DownscaleFilter
{
	DF_BOX,					// normaler Box-Filter
	DF_BOX_ALPHA_WEIGHTED	// Box-Filter mit Alphawichtung
};

// skaliert ein Bild auf die halbe Auflsung herunter
SDL_Surface* downscaleSurface2x(SDL_Surface* p_surface, DownscaleFilter filter);

// transformiert eine Shape-Definition
b2ShapeDef* transformShapeDef(const b2ShapeDef& shapeDef, const Vec2<float>& translation, const Vec2<float>& scaling);

// formatiert eine Zeitangabe in MM:SS.ss
const std::string formatTime(float t);

// entfernt Leerzeichen am Anfang
const std::string strip(const std::string& s);

// ersetzt alle Grobuchstaben durch Kleinbuchstaben
const std::string toLower(const std::string& s);

// liefert die nchste Zweierpotenz
unsigned int nextPow2(unsigned int x);

// liefert eine Kurve fr 0 <= x <= 1, die sanft von 0 nach 1 geht
// f'(0) = f'(1) = 0
float easeInEaseOut(float x);

// Konvertierung eines Objekts in einen String,
// wobei das Objekt den <<-Operator fr Streams anbieten muss
template<typename T> inline const std::string toString(const T& obj)
{
	std::ostringstream stream;
	stream << obj;
	return stream.str();
}

// Konvertierung eines Strings in ein Objekt,
// wobei das Objekt den >>-Operator fr Streams anbieten muss
template<typename T> inline const T fromString(const std::string& str,
											   const T& defaultValue = T())
{
	std::istringstream stream(str);
	T obj(defaultValue);
	stream >> obj;
	return obj;
}

// generiert automatische IDs fr Objekte
template<typename T> inline int autoID()
{
	static int nextID = 1;
	return nextID++;
}

// generiert automatische String-IDs fr Objekte
template<typename T> inline const std::string autoIDString(const std::string& prefix)
{
	std::ostringstream stream;
	stream << prefix << autoID<T>();
	return stream.str();
}

#endif
