/*****************************************************************/
/*      ____  ____  __    __    __  _______  ____  ___    ____   */
/*     / __ \/ __ \/ /   / /   /  |/  / __ \/ __ \/   |  / __ \  */
/*    / /_/ / / / / /   / /   / /|_/ / / / / /_/ / /| | / / / /  */
/*   / _, _/ /_/ / /___/ /___/ /  / / /_/ / _, _/ ___ |/ /_/ /   */
/*  /_/ |_|\____/_____/_____/_/  /_/\____/_/ |_/_/  |_/_____/    */
/*          ___ _   _ _  _   _   ___ ___   ___ __  _  __         */
/*         / __| | | | || | /_\ / __| __| |_  )  \/ |/  \        */
/*        | (_ | |_| | __ |/ _ \\__ \ _|   / / () | | () |       */
/*         \___|\___/|_||_/_/ \_\___/___| /___\__/|_|\__/        */
/*                                                               */
/*                    --- EIN TOLLES SPIEL ---                   */
/*                                                               */
/*         Jan Hapke, Wilhelm Mailnder, David Scherfgen         */
/*                                                               */
/*****************************************************************/

#ifndef VEC2_H_INCLUDED
#define VEC2_H_INCLUDED

// Template fr 2D-Vektoren mit Koordinaten beliebigen Typs
template<typename T> struct Vec2
{
	// Standardkonstruktor: initialisiert nichts
	Vec2();

	// Kopierkonstruktor-Template mit Typumwandlung
	template<typename U> Vec2(const Vec2<U>& toCopy);

	// Kopierkonstruktor zur Umwandlung von b2Vec2
	Vec2(const b2Vec2& toCopy);

	// initialisiert beide Koordinaten mit dem Wert v
	explicit Vec2(const T& v);

	// initialisiert den Vektor mit x und y
	Vec2(const T& x, const T& y);

	// initialisiert den Vektor mit p_xy[0] und p_xy[1]
	explicit Vec2(const T* p_xy);

	// testet, ob der Vektor ein Nullvektor ist
	bool isZero() const;

	// liefert die Vektorlnge
	const T length() const;

	// liefert das Quadrat der Vektorlnge
	const T lengthSq() const;

	// liefert eine normierte Kopie des Vektors
	const Vec2 normalizedCopy() const;

	// normiert den Vektor
	Vec2& normalize();

	// liefert einen Vektor gleicher Lnge, der senkrecht auf diesem Vektor steht
	const Vec2 perp() const;

	// Casting-Operator in konstanten Zeigertyp
	operator const T*() const;

	// Casting-Operator in Zeigertyp
	operator T*();

	// Casting-Operator in b2Vec2
	operator const b2Vec2() const;

	// Addition und Zuweisung
	Vec2& operator +=(const Vec2& rhs);

	// Subtraktion und Zuweisung
	Vec2& operator -=(const Vec2& rhs);

	// Multiplikation mit Skalar und Zuweisung
	Vec2& operator *=(const T& rhs);

	// komponentenweise Multiplikation mit Vektor und Zuweisung
	Vec2& operator *=(const Vec2& rhs);

	// Division durch Skalar und Zuweisung
	Vec2& operator /=(const T& rhs);

	// komponentenweise Division durch Vektor und Zuweisung
	Vec2& operator /=(const Vec2& rhs);

	// Negation
	Vec2 operator -() const;

	// Skalarprodukt
	static const T dot(const Vec2& lhs, const Vec2& rhs);

	// Winkel zwischen zwei Vektoren (Bogenma)
	static const T angle(const Vec2& lhs, const Vec2& rhs);

	// zuflliger Richtungsvektor der Lnge 1
	static const Vec2 randomDirection(MTRand& rng = MTRand::globalMTRand);

	T x;
	T y;
};

template<typename T> inline Vec2<T>::Vec2()
{

}

template<typename T> template<typename U> inline Vec2<T>::Vec2(const Vec2<U>& toCopy)
	: x(static_cast<T>(toCopy.x))
	, y(static_cast<T>(toCopy.y))
{

}

template<typename T> inline Vec2<T>::Vec2(const b2Vec2& toCopy)
	: x(static_cast<T>(toCopy.x))
	, y(static_cast<T>(toCopy.y))
{

}

template<typename T> inline Vec2<T>::Vec2(const T& v)
	: x(v)
	, y(v)
{

}

template<typename T> inline Vec2<T>::Vec2(const T& x,
										  const T& y)
	: x(x)
	, y(y)
{

}

template<typename T> inline Vec2<T>::Vec2(const T* p_xy)
	: x(p_xy[0])
	, y(p_xy[1])
{

}

template<typename T> inline bool Vec2<T>::isZero() const
{
	return x == static_cast<T>(0) && y == static_cast<T>(0);
}

template<typename T> inline const T Vec2<T>::length() const
{
	return sqrt(lengthSq());
}

template<typename T> inline const T Vec2<T>::lengthSq() const
{
	return x * x + y * y;
}

template<typename T> inline const Vec2<T> Vec2<T>::normalizedCopy() const
{
	return *this / length();
}

template<typename T> inline Vec2<T>& Vec2<T>::normalize()
{
	return *this = normalizedCopy();
}

template<typename T> inline const Vec2<T> Vec2<T>::perp() const
{
	return Vec2(-y, x);
}

template<typename T> inline Vec2<T>::operator const T*() const
{
	return &x;
}

template<typename T> inline Vec2<T>::operator T*()
{
	return &x;
}

template<typename T> inline Vec2<T>::operator const b2Vec2() const
{
	return b2Vec2(static_cast<float>(x), static_cast<float>(y));
}

template<typename T> inline Vec2<T> Vec2<T>::operator -() const
{
	return Vec2(-x, -y);
}

template<typename T> inline Vec2<T>& Vec2<T>::operator +=(const Vec2& rhs)
{
	x += rhs.x;
	y += rhs.y;
	return *this;
}

template<typename T> inline Vec2<T>& Vec2<T>::operator -=(const Vec2& rhs)
{
	x -= rhs.x;
	y -= rhs.y;
	return *this;
}

template<typename T> inline Vec2<T>& Vec2<T>::operator *=(const T& rhs)
{
	x *= rhs;
	y *= rhs;
	return *this;
}

template<typename T> inline Vec2<T>& Vec2<T>::operator *=(const Vec2& rhs)
{
	x *= rhs.x;
	y *= rhs.y;
	return *this;
}

template<typename T> inline Vec2<T>& Vec2<T>::operator /=(const T& rhs)
{
	x /= rhs;
	y /= rhs;
	return *this;
}

template<typename T> inline Vec2<T>& Vec2<T>::operator /=(const Vec2& rhs)
{
	x /= rhs.x;
	y /= rhs.y;
	return *this;
}

// Vektoraddition
template<typename T> inline Vec2<T> operator +(const Vec2<T>& lhs,
											   const Vec2<T>& rhs)
{
	return Vec2<T>(lhs.x + rhs.x, lhs.y + rhs.y);
}

// Vektorsubtraktion
template<typename T> inline Vec2<T> operator -(const Vec2<T>& lhs,
											   const Vec2<T>& rhs)
{
	return Vec2<T>(lhs.x - rhs.x, lhs.y - rhs.y);
}

// Skalar von links multipliziert mit Vektor
template<typename T> inline Vec2<T> operator *(const T& lhs,
											   const Vec2<T>& rhs)
{
	return Vec2<T>(lhs * rhs.x, lhs * rhs.y);
}

// Skalar von rechts multipliziert mit Vektor
template<typename T> inline Vec2<T> operator *(const Vec2<T>& lhs,
											   const T& rhs)
{
	return Vec2<T>(lhs.x * rhs, lhs.y * rhs);
}

// komponentenweise Vektormultiplikation
template<typename T> inline Vec2<T> operator *(const Vec2<T>& lhs,
											   const Vec2<T>& rhs)
{
	return Vec2<T>(lhs.x * rhs.x, lhs.y * rhs.y);
}

// Vektor dividiert durch Skalar
template<typename T> inline Vec2<T> operator /(const Vec2<T>& lhs,
											   const T& rhs)
{
	return Vec2<T>(lhs.x / rhs, lhs.y / rhs);
}

// komponentenweise Vektordivision
template<typename T> inline Vec2<T> operator /(const Vec2<T>& lhs,
											   const Vec2<T>& rhs)
{
	return Vec2<T>(lhs.x / rhs.x, lhs.y / rhs.y);
}

// Vergleichsoperator
template<typename T> inline bool operator ==(const Vec2<T>& lhs,
											 const Vec2<T>& rhs)
{
	return lhs.x == rhs.x && lhs.y == rhs.y;
}

// Kleiner-Operator (lexikografisch)
template<typename T> inline bool operator <(const Vec2<T>& lhs,
											const Vec2<T>& rhs)
{
	if(lhs.x != rhs.x) return lhs.x < rhs.x;
	return lhs.y < rhs.y;
}

// Stream-Ausgabe eines Vektors
template<typename T> inline std::ostream& operator <<(std::ostream& lhs,
													  const Vec2<T>& rhs)
{
	return lhs << '(' << rhs.x << ", " << rhs.y << ')';
}

// Stream-Eingabe eines Vektors,
// im Format (x, y)
template<typename T> inline std::istream& operator >>(std::istream& lhs,
													  Vec2<T>& rhs)
{
	// ffnende Klammer lesen
	char c = 0;
	lhs >> c;
	if(c != '(') lhs.clear(std::ios_base::badbit);

	// x-Koordinate, Komma und y-Koordinate lesen
	Vec2<T> result;
	lhs >> result.x;
	lhs >> c;
	if(c != ',') lhs.clear(std::ios_base::badbit);
	lhs >> result.y;

	// schlieende Klammer lesen
	lhs >> c;
	if(c != ')') lhs.clear(std::ios_base::badbit);

	if(lhs)
	{
		// Ergebnis kopieren
		rhs = result;
	}

	return lhs;
}

template<typename T> inline const T Vec2<T>::dot(const Vec2<T>& lhs,
												 const Vec2<T>& rhs)
{
	return lhs.x * rhs.x + lhs.y * rhs.y;
}

template<typename T> inline const T Vec2<T>::angle(const Vec2<T>& lhs,
												   const Vec2<T>& rhs)
{
	return acos(dot(lhs, rhs) / (lhs.length() * rhs.length()));
}

template<typename T> inline const Vec2<T> Vec2<T>::randomDirection(MTRand& rng)
{
	const T angle = rng.randomFloatExclusive(0.0f, M_2_PI_FLOAT);
	return Vec2(sin(angle), cos(angle));
}

// Standardvektortypen
typedef Vec2<int> Vec2i;
typedef Vec2<float> Vec2f;
typedef Vec2<double> Vec2d;

#endif
